#!/usr/bin/perl

#
# iscsi-portal
#
# Steffen Plotner, 2010
#
# Implementation of iSCSI SendTargets=All command (no CHAP is used/allowed/supported)
#
use strict;
use Data::Dumper;
use Getopt::Std;

use FindBin;
use lib "$FindBin::Bin";
use lib "$FindBin::Bin/lib";
use lib "$FindBin::Bin/etc";

use config;
use logutil;
use iscsiportal;
use iscsi;

#
# handle command line options
#
my %opts;
getopts('hvdi:lq:c:',\%opts);
if ( defined($opts{'h'}) )
{
        print <<EOH;
$0 [-h] [-v] [-d] [-c] -i iqn | -l | -q pgname

-h	help

-i	initiator name and perform sendtargets=all against all discovery portals

-l	list current portal group names and their associated discovery session ip address

-c	debug a hardcoded initiator problem
-v	verbose
-d	debug PDUs

EOH
	exit(0);
}

$cfg{'verbose'} = 0;
if ( defined($opts{'v'}) )
{
	$cfg{'verbose'} = 1;
}
$cfg{'debug'} = 0;
if ( defined($opts{'d'}) )
{
	$cfg{'debug'} = 1;
}

if ( defined($opts{'l'}) )
{
	portal_group_name_list();
}
elsif ( defined($opts{'i'}) )
{
	my $initiator_name = $opts{'i'};
	simulate_initiator($initiator_name);
}
elsif ( defined($opts{'c'}) )
{
	my $initiator_name = $opts{'c'};
	debug_initiator($initiator_name);
}
else
{
	logger_err("portal> $0 -h");
	exit(1);
}

exit(0);

#------------------------------------------------------------------------------
# list portal group names and their associated ip address (one per groupname only)
#------------------------------------------------------------------------------
sub portal_group_name_list
{
	foreach my $discovery_portal (keys %{$cfg{'discovery_portals'}} )
	{
		print $cfg{'discovery_portals'}{$discovery_portal}{'portal_group_name'};
		print " ";
		print $discovery_portal;
		print "\n";
	}
}

#------------------------------------------------------------------------------
# lookup discovery portal ip by portal group name
#------------------------------------------------------------------------------
sub portal_group_name_lookup
{
	my ($portal_group_name) = @_;

	foreach my $discovery_portal (keys %{$cfg{'discovery_portals'}} )
	{
		if ( $portal_group_name eq $cfg{'discovery_portals'}{$discovery_portal}{'portal_group_name'} )
		{
			return $discovery_portal;
		}
	}
	return undef;
}

#------------------------------------------------------------------------------
# simulate initiator iqn and perform sendtargets=all against all related
# target portal address based on the portal group name supplied
#------------------------------------------------------------------------------
sub simulate_initiator
{
	my ($initiator_name) = @_;

	#
	# enum all known portal addresses and use those to enumerate
	# what this initiator can reach (i.e. across all VLAN, a normal initiator would not do that(!))
	#
	foreach my $discovery_portal ( sort keys %{$cfg{'discovery_portals'}} )
	{
		print "portal: $discovery_portal ($cfg{'discovery_portals'}{$discovery_portal}{'portal_group_name'})\n";
		
		#if ( $discovery_portal eq '148.85.141.21' )
		#{
			my $discovered_targets = initiator_discover_targets($discovery_portal, $initiator_name);
			foreach my $target (keys %{$discovered_targets} )
			{
				print " target: $target\n";
			}
		#}
	}
}

sub debug_initiator
{
	my ($initiator_name) = @_;
	
 	my $targets = iscsi_initiator_sendtargets_all($initiator_name, "x.x.x.x", 3260);
}
